#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <err.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <signal.h>
#include <syslog.h>
#include <varargs.h>
#include <sys/wait.h>
#include <pwd.h>


#define MINSAFEUID 100 	/* the UID of the lowest user that this program will run as.*/
#define BATTERY 10   
#define UTILITY 13      /* actually, 12 means charging and 13 is fully charged */ 
#define NOBODY          "nobody"        /* Nobody pwnam */
#define SYSLOGFACILITY  LOCAL1

/*********************************************************

(c) Sentex Communications 2003. Written by Keith Winter and by Mike Tancsa

 * This driver is based on a previous driver by Invertex, for which they
 * requested:  Please send any comments, feedback, bug-fixes, or feature
 * requests to software@invertex.com.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


Trival daemon that can run in the background to monitor
an APC 500 ES and RS series

Tested with
# usbdevs -v
Controller /dev/usb0:
addr 1: self powered, config 1, UHCI root hub(0x0000), VIA(0x0000), rev 1.00
 port 1 powered
 port 2 addr 2: low speed, self powered, config 1, Back-UPS ES 500 FW:801.e5.D USB FW:e5(0x0002), APC(0x051d), rev 1.06

Not much functionality to it, but it will tell you if its on battery or on street power. 
It also tells you how much battery is left and an estimated time to live. However, I dont find
that accurate at all.  Send it a SIGUP or USR1 and it logs power to local1. It also sets the 
proctitle to let you know the battery strength.  It is also hard coded to run
/usr/local/sbin/onbattery.sh
/usr/local/sbin/onutility.sh

It passes as an arg the amount of power left to those shell scripts


*********************************************************/

int execommand(char *shellCommand);

static const char rcsid[] = "version 1.0. 2003/09/15 mdt usb-apcd-es";
static char lastStrength[256] = "00";
static void handleSignal(int signal);
static unsigned char buff[ 128 ];

int
main( int argc, char **argv )
{

	char cmd[512];
        int fd;
        int count;
        int i;
        int loop;
	int onbattery=0;
	int previousState = -1;
	int currentState  = -1;
	struct passwd *pwent;
	pid_t pid;
	


	/* when we get a SigUP spit out to syslog the current battery level */

	signal(SIGHUP,  handleSignal);
	signal(SIGUSR1,  handleSignal);


	/* usage is ./apc-es-upsd <user> <devicename> <on battery program> <on utility program> */
	if (argc != 5) {
		fprintf(stderr, "usage: ./apc-es-upsd <user> <devicename> <on battery program> <on utility program>\n");
		exit(1);
	}

	/* make sure we dont try and read some silly device name */
	if (strlen(argv[2]) > 255 ) {
		fprintf(stderr, "I can only understand device names less than 256 chars long \n");
		exit(1);
	}
        fd = open( argv[2], O_RDONLY );
        if( fd < 0 ) {
                perror( "device open error " );
                return -1;
        }


	/* make sure the user exists */
	if (!(pwent = getpwnam(argv[1]))) 
    	{
        	fprintf(stderr, "The userid you gave me (%s) does not exist!\n",argv[1]);
        	exit(1);
    	}
	/*  make sure its something reasonably high and not special */
	if (pwent->pw_uid < MINSAFEUID) {
		fprintf(stderr, "please use a uid > 100\n");
		exit(1);
	}
	/*  Lets drop our privs as we dont need to live as root once we have opened the device */ 
    	if (setuid(pwent->pw_uid) == -1)
    	{
		fprintf(stderr, "Cant drop privledges \n");
        	exit(1);
    	}
	
	
	/*  Lets become a daemon at this point and run in the background */
	pid = fork();
	if(pid < 0)
	{
		fprintf(stderr, "Cant fork and become a daemon for some reason ? \n");
        	exit(1);
	}
	else if(pid > 0)
		    exit(0); 
	if(setsid() < 0)
	{
		fprintf(stderr, "Cant become a daemon for some reason ? \n");
        	exit(1);
	}
	/* now we're a daemon */


        loop = 0;
        do {
                count = read( fd, buff, sizeof( buff ) );
                for( i = 0; i < count; i++ )
                fflush( stdout );
                switch( buff[ 0 ] ) {
			case 22   :
				/*  Battery status 10 = on battery   12,13 = utility power */
				if (buff[1] == BATTERY) { 
					onbattery =1;
					if (buff[1] == previousState){
						#ifdef DEBUG
						printf("no change still on battery\n");
						#endif
					}
					else {
						#ifdef DEBUG
						printf("* now on battery! %d %d**\n" , buff[1],previousState); 
						#endif
						syslog(LOG_NOTICE|LOG_LOCAL1, "ON BATTERY: %s",lastStrength);
						sprintf(cmd,"/usr/local/sbin/onbattery.sh \"%s\"",lastStrength);
						execommand(cmd);
						
					}
				}
				if (buff[1] > BATTERY) { 
					onbattery =0;
					if (buff[1] == previousState){
						#ifdef DEBUG
						printf("no change still on utility\n");
						#endif

					}
					else {
						#ifdef DEBUG
						printf("** now back on Utility! **\n");
						#endif
						sprintf(cmd,"/usr/local/sbin/onutility.sh \"%s\"",lastStrength);
						syslog(LOG_NOTICE|LOG_LOCAL1, "BACK ON UTILITY: %s",lastStrength);
						execommand(cmd);
					}
				}

				previousState = buff[1];
				currentState  = buff[1];

 
                        case 0x06 :
                                /* printf( "0x06: Lifetime - %d mins\n", buff[ 3 ] ); */
                                break;
                        case 12 :
				sprintf(lastStrength," Battery strength Level %3d%% ETTL %3d min\n ", buff[ 1 ], buff[3]); 
				setproctitle("battery=%d%%", buff[1]);
				if (onbattery) {
					syslog(LOG_NOTICE|LOG_LOCAL1, "Battery strength Level %3d%% ETTL %3d min", buff[ 1 ], buff[3]); 
				}
                                break;
                        default :
                                break;
                }

		if (loop++ >=5) {
			loop=0;

			/* Lets get the temp readings */


		}
		

        } while( count >= 1 );
}

static void handleSignal(int signal) {

	if(lastStrength) {
		if (signal == SIGHUP)
		{
			
        		syslog(LOG_INFO|LOG_LOCAL1,"signal: %s",lastStrength);
		}
		else
		{
        		syslog(LOG_INFO|LOG_LOCAL1,"signal: %d %d %d %s",buff[0],buff[1],buff[2],lastStrength);

		}
			
	}
}



/* Subroutine to execute shell script command */

int execommand(char *shellCommand)
{
        system(shellCommand);
	return 0;
        switch(fork()) {
        case 0:
                system(shellCommand);
                exit(0);
        case -1:
                errx(1, "unable to execute shell command: %s", strerror(errno));
                break;
        default:
                break;
        }
}

